# FILE: mmlaunch.tcl
#
# An application which presents an interface for launching and 
# interconnecting other OOMMF applications.
#
# Last modified on: $Date: 2000/08/24 22:42:37 $
# Last modified by: $Author: donahue $

# Support libraries
package require Ow
package require Net
package require Oc 1.1.0.1
if {[catch {package require Tk 8}]} {
    package require Tk 4.1
}
wm withdraw .

Oc_Main SetAppName mmLaunch
Oc_Main SetVersion 1.2.0.0
regexp \\\044Date:(.*)\\\044 {$Date: 2000/08/24 22:42:37 $} _ date
Oc_Main SetDate [string trim $date]
# regexp \\\044Author:(.*)\\\044 {$Author: donahue $} _ author
# Oc_Main SetAuthor [Oc_Person Lookup [string trim $author]]
Oc_Main SetAuthor [Oc_Person Lookup dgp]
Oc_Main SetHelpURL [Oc_Url FromFilename [file join [file dirname \
        [file dirname [file dirname [Oc_DirectPathname [info \
        script]]]]] doc userguide userguide \
	OOMMF_Launcher_Control_Inte.html]]
Oc_CommandLine Parse $argv

set menubar .mb
foreach {fmenu hmenu} [Ow_MakeMenubar . $menubar File Help] {}
$fmenu add command -label "Exit" -underline 1 -command exit
Ow_StdHelpMenu $hmenu

# Host checkbutton frame
set hframe [frame .omfHosts -bd 4 -relief ridge]
set hpanel [frame .omfPanel -relief groove]

set menuwidth [Ow_GuessMenuWidth $menubar]
set brace [canvas .brace -width $menuwidth -height 0 -borderwidth 0 \
        -highlightthickness 0]
pack $brace -side top

pack $hframe -side left -fill both -expand 1
pack $hpanel -side left -fill both -expand 1

wm title . [Oc_Main GetInstanceName]
wm iconname . [wm title .]
Ow_SetIcon .
wm deiconify .

if {[package vcompare [package provide Tk] 8] >=0 \
	&& [string match windows $tcl_platform(platform)]} {
    # Windows doesn't size Tcl 8.0 menubar cleanly
    Oc_DisableAutoSize .
    wm geometry . "${menuwidth}x0"
    update
    wm geometry . {}
    Oc_EnableAutoSize .
    update
}

proc DisplayHostButtons {} {
    global hframe hpanel
    # Destroy existing buttons
    foreach w [winfo children $hframe] {
        destroy $w
    }
    set displayedhost 0
    foreach host [Net_Host Instances] {
        if {[$host Ready]} {
            set btn [checkbutton $hframe.w$host -text [$host Cget -hostname] \
                -anchor w \
                -variable [$host GlobalName btnvar] \
                -command [list $host ToggleGui $hpanel]]
            pack $btn -side top -fill x
            set displayedhost 1
        }
    }
    wm geometry . {}
}
Oc_EventHandler New _ Net_Host Ready DisplayHostButtons
Oc_EventHandler New _ Net_Host Delete DisplayHostButtons

# When any account becomes ready, get its threads
Oc_EventHandler New _ Net_Account Ready [list GetThreads %object]

proc GetThreads { acct } {
    set qid [$acct Send threads]
    Oc_EventHandler New _ $acct Reply$qid [list GetThreadsReply $acct $qid] \
        -groups [list $acct GetThreads-$qid]
    Oc_EventHandler New _ $acct Timeout$qid \
	[list GetThreadsTimeout $acct $qid] \
        -groups [list $acct GetThreads-$qid]
}
proc GetThreadsReply { acct qid } {
    Oc_EventHandler DeleteGroup GetThreads-$qid
    # Set up to receive NewThread messages, but only one handler per account
    Oc_EventHandler DeleteGroup GetThreadsReply-$acct
    Oc_EventHandler New _ $acct Readable [list HandleAccountTell $acct] \
            -groups [list $acct GetThreadsReply-$acct]
    set threads [$acct Get]
    Oc_Log Log "Received thread list: $threads" status
    # Create a Net_Thread instance for each element of the returned
    # thread list
    if {![lindex $threads 0]} {
        foreach triple [lrange $threads 1 end] {
            Net_Thread New _ -hostname [$acct Cget -hostname] \
                    -accountname [$acct Cget -accountname] \
                    -pid [lindex $triple 1]
        }
    }
}
proc GetThreadsTimeout { acct qid } {
    Oc_EventHandler DeleteGroup GetThreads-$qid
    set msg "Timed out waiting for thread list"
    error $msg $msg
}

# Detect and handle NewThread message from account server
proc HandleAccountTell { acct } {
    set message [$acct Get]
    switch -exact -- [lindex $message 0] {
        newthread {
            NewThread $acct [lindex $message 1]
        }
        default {
          Oc_Log Log "Bad message from account $acct:\n\t$message" status
        }
    }
}

# There's a new thread with id $id, create corresponding local instance
proc NewThread { acct id } {
    Net_Thread New _ -hostname [$acct Cget -hostname] \
            -accountname [$acct Cget -accountname] -pid $id
}

# Create initial host list -- try to connect to account servers
set hostList localhost
Oc_Option Get LaunchInterface hostList hostList
foreach host $hostList {
    if {![string match $host* [info hostname]]} {
        Net_Host New _ -hostname $host
        Net_Account New _ -hostname $host
    }
}
if {[string match localhost [info hostname]]} {
    Net_Host New _ -hostname localhost
    Net_Account New _ -hostname localhost
}

