/* FILE: simpledemag.h            -*-Mode: c++-*-
 *
 * Average H demag field across rectangular cells.  Simple
 * implementation that does not take advantage of any memory
 * saving or calculation speed improvements possible using
 * symmetries in interaction tensor or realness of data in
 * spatial domain.
 *
 */

#ifndef _OXS_SIMPLEDEMAG
#define _OXS_SIMPLEDEMAG

#include "key.h"
#include "energy.h"
#include "fft.h"
#include "mesh.h"
#include "meshvalue.h"
#include "simstate.h"
#include "threevector.h"
#include "rectangularmesh.h"

/* End includes */

class Oxs_SimpleDemag:public Oxs_Energy {
private:
  UINT4m xdim,ydim,zdim,totalsize;  // Natural size
  UINT4m pxdim,pydim,pzdim,ptotalsize;  // Padded size
  Oxs_Key<Oxs_RectangularMesh> meshkey;
  // The A## arrays hold demag coefficients, transformed into
  // frequency domain.  These are held long term.  Mx, My, Mz
  // and Hcomp are used as temporary space to convert M into
  // H.
  MyComplex *A00,*A01,*A02,*A11,*A12,*A22;
  MyComplex *Mx,*My,*Mz,*Hcomp;
  FFT3D fft; // All transforms are same size, so we need
  /// only one FFT3D object.

  UINT4m NextPowerOfTwo(UINT4m n) const;  // Helper function
  void ReleaseMemory();
public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  virtual BOOL Init();
  Oxs_SimpleDemag(const char* name,     // Child instance id
		  Oxs_Director* newdtr, // App director
		  Tcl_Interp* safe_interp, // Safe interpreter
		  const char* argstr);  // MIF input block parameters
  virtual ~Oxs_SimpleDemag() { ReleaseMemory(); }

  virtual void GetEnergyAndField(const Oxs_SimState& state,
                                 Oxs_MeshValue<REAL8m>& energy,
                                 Oxs_MeshValue<ThreeVector>& field
                                 ) const;
};


#endif // _OXS_SIMPLEDEMAG

